package javax.lang.model.util;

import javax.lang.model.element.TypeParameterElement;
import javax.lang.model.element.ExecutableElement;
import javax.lang.model.element.VariableElement;
import javax.lang.model.element.TypeElement;
import javax.lang.model.element.ElementKind;
import javax.lang.model.element.PackageElement;
import javax.lang.model.SourceVersion;
import javax.annotation.processing.SupportedSourceVersion;

@SupportedSourceVersion(SourceVersion.RELEASE_6)
public class ElementKindVisitor6<R, P> extends SimpleElementVisitor6<R, P>
{
    protected ElementKindVisitor6() {
        super(null);
    }
    
    protected ElementKindVisitor6(R r) {
        super(r);
    }
    
    @Override
    public R visitPackage(PackageElement packageElement, P p2) {
        assert packageElement.getKind() == ElementKind.PACKAGE : "Bad kind on PackageElement";
        return this.defaultAction(packageElement, p2);
    }
    
    @Override
    public R visitType(TypeElement obj, P p2) {
        ElementKind kind = obj.getKind();
        switch (kind) {
            case ANNOTATION_TYPE: {
                return this.visitTypeAsAnnotationType(obj, p2);
            }
            case CLASS: {
                return this.visitTypeAsClass(obj, p2);
            }
            case ENUM: {
                return this.visitTypeAsEnum(obj, p2);
            }
            case INTERFACE: {
                return this.visitTypeAsInterface(obj, p2);
            }
            default: {
                throw new AssertionError("Bad kind " + kind + " for TypeElement" + obj);
            }
        }
    }
    
    public R visitTypeAsAnnotationType(TypeElement typeElement, P p2) {
        return this.defaultAction(typeElement, p2);
    }
    
    public R visitTypeAsClass(TypeElement typeElement, P p2) {
        return this.defaultAction(typeElement, p2);
    }
    
    public R visitTypeAsEnum(TypeElement typeElement, P p2) {
        return this.defaultAction(typeElement, p2);
    }
    
    public R visitTypeAsInterface(TypeElement typeElement, P p2) {
        return this.defaultAction(typeElement, p2);
    }
    
    @Override
    public R visitVariable(VariableElement obj, P p2) {
        ElementKind kind = obj.getKind();
        switch (kind) {
            case ENUM_CONSTANT: {
                return this.visitVariableAsEnumConstant(obj, p2);
            }
            case EXCEPTION_PARAMETER: {
                return this.visitVariableAsExceptionParameter(obj, p2);
            }
            case FIELD: {
                return this.visitVariableAsField(obj, p2);
            }
            case LOCAL_VARIABLE: {
                return this.visitVariableAsLocalVariable(obj, p2);
            }
            case PARAMETER: {
                return this.visitVariableAsParameter(obj, p2);
            }
            case RESOURCE_VARIABLE: {
                return this.visitVariableAsResourceVariable(obj, p2);
            }
            default: {
                throw new AssertionError("Bad kind " + kind + " for VariableElement" + obj);
            }
        }
    }
    
    public R visitVariableAsEnumConstant(VariableElement variableElement, P p2) {
        return this.defaultAction(variableElement, p2);
    }
    
    public R visitVariableAsExceptionParameter(VariableElement variableElement, P p2) {
        return this.defaultAction(variableElement, p2);
    }
    
    public R visitVariableAsField(VariableElement variableElement, P p2) {
        return this.defaultAction(variableElement, p2);
    }
    
    public R visitVariableAsLocalVariable(VariableElement variableElement, P p2) {
        return this.defaultAction(variableElement, p2);
    }
    
    public R visitVariableAsParameter(VariableElement variableElement, P p2) {
        return this.defaultAction(variableElement, p2);
    }
    
    public R visitVariableAsResourceVariable(VariableElement variableElement, P p2) {
        return this.visitUnknown(variableElement, p2);
    }
    
    @Override
    public R visitExecutable(ExecutableElement obj, P p2) {
        ElementKind kind = obj.getKind();
        switch (kind) {
            case CONSTRUCTOR: {
                return this.visitExecutableAsConstructor(obj, p2);
            }
            case INSTANCE_INIT: {
                return this.visitExecutableAsInstanceInit(obj, p2);
            }
            case METHOD: {
                return this.visitExecutableAsMethod(obj, p2);
            }
            case STATIC_INIT: {
                return this.visitExecutableAsStaticInit(obj, p2);
            }
            default: {
                throw new AssertionError("Bad kind " + kind + " for ExecutableElement" + obj);
            }
        }
    }
    
    public R visitExecutableAsConstructor(ExecutableElement executableElement, P p2) {
        return this.defaultAction(executableElement, p2);
    }
    
    public R visitExecutableAsInstanceInit(ExecutableElement executableElement, P p2) {
        return this.defaultAction(executableElement, p2);
    }
    
    public R visitExecutableAsMethod(ExecutableElement executableElement, P p2) {
        return this.defaultAction(executableElement, p2);
    }
    
    public R visitExecutableAsStaticInit(ExecutableElement executableElement, P p2) {
        return this.defaultAction(executableElement, p2);
    }
    
    @Override
    public R visitTypeParameter(TypeParameterElement typeParameterElement, P p2) {
        assert typeParameterElement.getKind() == ElementKind.TYPE_PARAMETER : "Bad kind on TypeParameterElement";
        return this.defaultAction(typeParameterElement, p2);
    }
}
